// Learn cc.Class:
//  - [Chinese] https://docs.cocos.com/creator/manual/zh/scripting/class.html
//  - [English] http://docs.cocos2d-x.org/creator/manual/en/scripting/class.html
// Learn Attribute:
//  - [Chinese] https://docs.cocos.com/creator/manual/zh/scripting/reference/attributes.html
//  - [English] http://docs.cocos2d-x.org/creator/manual/en/scripting/reference/attributes.html
// Learn life-cycle callbacks:
//  - [Chinese] https://docs.cocos.com/creator/manual/zh/scripting/life-cycle-callbacks.html
//  - [English] https://www.cocos2d-x.org/docs/creator/manual/en/scripting/life-cycle-callbacks.html

/************
 * 当带透明边框的sprite 渲染到纹理时，其透明边框部分会逐渐增大
 * 测试时在web下，鼠标点击选中视口区域，重复按键盘 A 键就能重现
 * *******/
cc.Class({
    extends: cc.Component,

    properties: {},

    start () {
        // 查找渲染sp的摄像机
        this.uiCamera = cc.find("Canvas/spCamera").getComponent(cc.Camera);
        this.uiCamera.active = false;

        // 测试的sprite，它带有透明边框。每次按下键盘 A 键，会将起通过 uiCamera 渲染到纹理，然后再用此纹理显示，图像会缩小
        this.uiPaper = cc.find("Canvas/sp").getComponent(cc.Sprite);

        // 生成两张 rendertexture，一张使用一张渲染，每次相互切换
        this.rt = [{ bUse: false, txtr: new cc.RenderTexture() }, { bUse: false, txtr: new cc.RenderTexture() }];

        // 挂载键盘事件
        cc.systemEvent.on(cc.SystemEvent.EventType.KEY_UP, this.TestClick.bind(this), this);
    },

    // update (dt) {},
    
    /// 
    TestClick() {
        cc.log("Key up", event.keyCode);

        if (event.keyCode == 65) {    // A
            // 为渲染到纹理做准备，显示摄像机，翻转节点，获取节点大小
            this.uiCamera.active = true;
            // 翻转node用于渲染到纹理
            this.uiPaper.node.setScale(1, -1);
            var sz = this.uiPaper.node.getContentSize();

            // 切换rendertexture
            var rt = this.rt[0];
            if (rt.bUse) { rt.bUse = false; rt = this.rt[1]; }
            else { this.rt[1].bUse = false; }
            rt.bUse = true;
            
            // 如果截图内容中不包含 Mask 组件，可以不用传递第三个参数
            rt.txtr.initWithSize(sz.width, sz.height, cc.game._renderContext.STENCIL_INDEX8)
            this.uiCamera.targetTexture = rt.txtr;
            // 渲染一次摄像机，即更新一次内容到 RenderTexture 中
            this.uiCamera.render();

            // 修改sp的纹理
            this.uiPaper.spriteFrame = new cc.SpriteFrame(rt.txtr);
            cc.log("ftest2 ----当前节点尺寸------- ", this.uiPaper.node.getContentSize());
            
            // 还原场景
            this.uiPaper.node.setScale(1, 1);
            this.uiCamera.targetTexture = null;
            this.uiCamera.active = false;
        }
    },

});
